@@ -1,21 +1,8 @@
 /* This file is part of the KDE project
-   Copyright (C) 1998, 1999 Simon Hausmann <hausmann@kde.org>
-   Copyright (C) 2016 David Faure <faure@kde.org>
+    SPDX-FileCopyrightText: 1998, 1999 Simon Hausmann <hausmann@kde.org>
+    SPDX-FileCopyrightText: 2016 David Faure <faure@kde.org>
 
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: GPL-2.0-or-later
 */
 
 #include "konqapplication.h"
@@ -41,6 +28,7 @@
 #include <QDir>
 #include <QDirIterator>
 #include <QStandardPaths>
+#include <QProcess>
 
 #include <KDBusService>
 #include <QCommandLineParser>
@@ -289,8 +277,12 @@ extern "C" Q_DECL_EXPORT int kdemain(int argc, char **argv)
         }
     }
 
+    QString programName = QApplication::applicationFilePath();
+
     const int ret = app.exec();
 
+    bool alwaysPreload = KonqSettings::alwaysHavePreloaded();
+
     // Delete all KonqMainWindows, so that we don't have
     // any parts loaded when KLibLoader::cleanUp is called.
     // (and Qt doesn't delete WA_DeleteOnClose widgets on exit anyway :()
@@ -305,5 +297,9 @@ extern "C" Q_DECL_EXPORT int kdemain(int argc, char **argv)
 
     KonqClosedWindowsManager::destroy();
 
+    if (alwaysPreload) {
+        QProcess::startDetached(programName, {"--preload"});
+    }
+
     return ret;
 }
