@@ -27,7 +27,6 @@ class WebSslInfo;
 class WebEnginePart;
 class KPasswdServerClient;
 class WebEngineWallet;
-class WebEnginePartCertificateErrorDlg;
 
 class WebEnginePage : public QWebEnginePage
 {
@@ -50,14 +49,12 @@ public:
      */
     void setSslInfo (const WebSslInfo &other);
 
-    void download(const QUrl &url, const QString &mimetype, bool newWindow = false);
+    void download(QWebEngineDownloadItem *it, bool newWindow = false);
 
     void requestOpenFileAsTemporary(const QUrl &url, const QString &mimeType = "", bool newWindow = false);
 
     void setStatusBarText(const QString &text);
 
-    WebEngineWallet* wallet() const {return m_wallet;}
-
     /**
     * @brief Tells the page that the part has requested to load the given URL
     *
@@ -67,6 +64,12 @@ public:
     */
     void markUrlAsRequestedByApp(const QUrl &url){m_urlRequestedByApp = url;}
 
+    /**
+     * @brief Sets the webengine part to be used by this object.
+     * @param part the part
+     */
+    void setPart(WebEnginePart *part);
+
 Q_SIGNALS:
     /**
      * This signal is emitted whenever a user cancels/aborts a load resource
@@ -83,12 +86,6 @@ protected:
      */
     WebEnginePart* part() const;
 
-    /**
-     * Sets the webengine part to be used by this object.
-     * @internal
-     */
-    void setPart(WebEnginePart*);
-
     /**
      * Reimplemented for internal reasons, the API is not affected.
      * @internal
@@ -134,15 +131,27 @@ private:
     void setPageJScriptPolicy(const QUrl& url);
 
     /**
-     * @brief Function called in response to the user closing the certificate error dialog
+     * @brief Function called when the part is forced to save an URL to disk.
      *
-     * Depending on the user's choice, this function will instruct the page to ignore the error or not
-     * and, if the user chose to forever ignore the error, it'll record this decision in the configuration
-     * file.
+     * This function should never be needed because the URL should be handled by the application itself.
+     * However, there can be cases in which this doesn't happen, in particular
+     * if there's something wrong with the system configuration and the application asks the part to
+     * handle something it can't display. In this case, it would work as follow:
+     * - the application asks the part to display the URL
+     * - the part can't display the URL, so a download is triggered, causing the openUrlRequest signal
+     *   to be emitted
+     * - the application receives the signal and decides that the part should handle the URL
+     * - the part can't display the URL and triggers a download
+     * - endless loop
      *
-     * @param dlg the certificate error dialog
+     * To avoid this situation, inside download(), the part checks whether it was asked to handle
+     * the URL by the application. In that case, it doesn't emit the openUrlRequest signal
+     * but calls this function to directly download the file to disk.
+     *
+     * @param it the item describing the download
      */
-    void handleCertificateError(WebEnginePartCertificateErrorDlg *dlg);
+     void saveUrlToDisk(QWebEngineDownloadItem *it);
+     void saveUrlUsingKIO(const QUrl &origUrl, const QUrl &destUrl);
 
 private:
     enum WebEnginePageSecurity { PageUnencrypted, PageEncrypted, PageMixed };
@@ -154,7 +163,6 @@ private:
     QPointer<WebEnginePart> m_part;
 
     QScopedPointer<KPasswdServerClient> m_passwdServerClient;
-    WebEngineWallet *m_wallet;
 
     /**
     * @brief The last URL that the application explicitly asked this part to open
@@ -207,7 +215,6 @@ private:
     KParts::WindowArgs m_windowArgs;
     WebWindowType m_type;
     bool m_createNewWindow;
-    WebEngineWallet* m_wallet;
 };
 
 #endif // WEBENGINEPAGE_H
